from openfisca_us.model_api import *


class meets_medicaid_income_threshold(Variable):
    value_type = bool
    entity = Person
    definition_period = YEAR
    label = "Meets Medicaid income threshold"
    documentation = "Whether the person meets the Medicaid income threshold given their state, age, and family structure."

    def formula(person, period, parameters):
        income = person.spm_unit("medicaid_gross_income", period)
        fpg = person.spm_unit("spm_unit_fpg", period)
        fpg_income_threshold = person("medicaid_income_threshold", period)
        income_share_of_fpg = income / fpg
        return income_share_of_fpg <= fpg_income_threshold



from openfisca_us.model_api import *


class spm_unit_taxes(Variable):
    value_type = float
    entity = SPMUnit
    label = "Taxes"
    definition_period = YEAR
    unit = USD

    def formula(spm_unit, period, parameters):
        SPMU_COMPONENTS = [
            "spm_unit_fica",
            "spm_unit_federal_tax",
            "spm_unit_state_tax",
        ]
        return add(spm_unit, period, SPMU_COMPONENTS)



from openfisca_us.model_api import *


class is_ssi_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    definition_period = YEAR
    documentation = "Eligibility for Supplemental Security Income"
    label = "SSI eligibility"



from openfisca_us.model_api import *


class ctc_refundable_individual_maximum(Variable):
    value_type = float
    entity = Person
    label = "Maximum refundable CTC"
    unit = "currency-USD"
    documentation = "The maximum refundable CTC for this person."
    definition_period = YEAR
    reference = (
        "https://www.law.cornell.edu/uscode/text/26/24#a",
        "https://www.law.cornell.edu/uscode/text/26/24#h",
        "https://www.law.cornell.edu/uscode/text/26/24#i",
    )

    def formula(person, period, parameters):
        return person("ctc_individual_maximum", period)

    def formula_2018(person, period, parameters):
        amount = person("ctc_individual_maximum", period)
        ctc = parameters(period).irs.credits.child_tax_credit
        return min_(amount, ctc.refundable.individual_max)

    def formula_2021(person, period, parameters):
        return person("ctc_child_individual_maximum", period)

    formula_2022 = formula_2018

    formula_2026 = formula


class ctc_limiting_tax_liability(Variable):
    value_type = float
    entity = TaxUnit
    label = "CTC-limiting tax liability"
    unit = "currency-USD"
    documentation = "The tax liability used to determine the maximum amount of the non-refundable CTC."
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        non_refundable_credits = parameters(period).irs.credits.non_refundable
        total_credits = sum(
            [
                tax_unit(credit, period)
                for credit in non_refundable_credits
                if credit != "non_refundable_ctc"
            ]
        )
        return tax_unit("income_tax_before_credits", period) - total_credits


class refundable_ctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "Refundable CTC"
    unit = "currency-USD"
    documentation = "The portion of the Child Tax Credit that is refundable."
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/24#d"

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        maximum_refundable_ctc = min_(
            tax_unit.sum(person("ctc_refundable_individual_maximum", period)),
            tax_unit("child_tax_credit", period),
        )

        liability = tax_unit("ctc_limiting_tax_liability", period)
        ctc = parameters(period).irs.credits.child_tax_credit
        earnings_over_threshold = max_(
            0,
            tax_unit.sum(person("earned_income", period))
            - ctc.refundable.phase_in.threshold,
        )
        relevant_earnings = (
            earnings_over_threshold * ctc.refundable.phase_in.rate
        )

        eitc = tax_unit("eitc", period)
        social_security_tax = tax_unit("social_security_taxes", period)
        social_security_excess = max_(0, social_security_tax - eitc)

        phased_in_amount = max_(relevant_earnings, social_security_excess)

        capped_phase_in = min_(phased_in_amount, liability)
        return min_(capped_phase_in, maximum_refundable_ctc)

    def formula_2021(tax_unit, period, parameters):
        return aggr(tax_unit, period, ["ctc_refundable_individual_maximum"])

    formula_2022 = formula


class non_refundable_ctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "Non-refundable CTC"
    unit = "currency-USD"
    documentation = (
        "The portion of the Child Tax Credit that is not refundable."
    )
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        return tax_unit("child_tax_credit", period) - tax_unit(
            "refundable_ctc", period
        )



from openfisca_us.model_api import *


class snap_max_allotment(Variable):
    value_type = float
    entity = SPMUnit
    definition_period = YEAR
    documentation = "Maximum SNAP allotment for SPM unit, based on the state group and household size."
    label = "SNAP maximum allotment"
    unit = USD

    def formula(spm_unit, period, parameters):
        max_allotments = parameters(period).usda.snap.max_allotment.main
        state_group = spm_unit.household("state_group_str", period)
        household_size = spm_unit("spm_unit_size", period)
        return max_allotments[state_group][household_size] * 12



from openfisca_us.model_api import *


class medicaid(Variable):
    value_type = float
    entity = SPMUnit
    definition_period = YEAR
    documentation = "Estimated benefit amount from Medicaid"
    label = "Medicaid benefit"
    unit = USD



from openfisca_us.model_api import *


class social_security_taxes(Variable):
    value_type = float
    entity = TaxUnit
    label = "Social security taxes"
    unit = "currency-USD"
    documentation = "Total employee-side social security taxes"
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        employee_payroll_tax = 0.5 * tax_unit("ptax_was", period)
        self_employed_tax = tax_unit("c03260", period)
        unreported_payroll_tax = aggr(tax_unit, period, ["e09800"])
        excess_payroll_tax_withheld = aggr(tax_unit, period, ["e11200"])
        return (
            employee_payroll_tax
            + self_employed_tax
            + unreported_payroll_tax
            - excess_payroll_tax_withheld
        )



from openfisca_us.model_api import *


class ctc_child_individual_maximum(Variable):
    value_type = float
    entity = Person
    label = "CTC maximum amount (child)"
    unit = "currency-USD"
    documentation = "The CTC entitlement in respect of this person as a child."
    definition_period = YEAR
    reference = (
        "https://www.law.cornell.edu/uscode/text/26/24#a",
        "https://www.law.cornell.edu/uscode/text/26/24#h",
        "https://www.law.cornell.edu/uscode/text/26/24#i",
    )

    def formula(person, period, parameters):
        ctc = parameters(period).irs.credits.child_tax_credit
        age = person("age", period)
        is_child = age <= ctc.child.max_age
        return is_child * ctc.child.amount

    def formula_2021(person, period, parameters):
        ctc = parameters(period).irs.credits.child_tax_credit
        age = person("age", period)
        is_child = age <= ctc.child.max_age
        is_young_child = age <= ctc.child.young.max_age
        return (
            is_young_child * ctc.child.young.increase
            + is_child * ctc.child.amount
        )

    formula_2022 = formula



from openfisca_us.model_api import *


class is_tax_unit_head(Variable):
    value_type = bool
    entity = Person
    label = "Head of tax unit"
    definition_period = ETERNITY

    def formula(person, period, parameters):
        # Establish basic conditions
        eligible = person("age", period) >= 18

        # Of those who meet the criteria, select the first person defined
        rank = person.tax_unit.members_position * eligible
        highest_rank = person.tax_unit.sum(rank)
        return eligible & (rank == highest_rank)



from openfisca_us.model_api import *


class school_meal_subsidy(Variable):
    value_type = float
    entity = SPMUnit
    definition_period = YEAR
    label = "School meal subsidy"
    unit = USD
    documentation = "Total school meal subsidy entitlement"

    def formula(spm_unit, period, parameters):
        # Get state group and tier (based on poverty ratio) for SPM unit.
        state_group = spm_unit.household("state_group_str", period)
        tier = spm_unit("school_meal_tier", period).decode_to_str()
        # Get parameters.
        p_school_meals = parameters(period).usda.school_meals
        p_amount = p_school_meals.amount
        # Get NSLP and SBP per child for each SPM unit.
        nslp_per_child = p_amount.nslp[state_group][tier]
        sbp_per_child = p_amount.sbp[state_group][tier]
        # Add NSLP and SBP.
        school_meal_subsidy_per_child = nslp_per_child + sbp_per_child
        # Multiply by number of school days in the year and number of children
        # in school.
        children = spm_unit.sum(spm_unit.members("is_in_school", period))
        return (
            school_meal_subsidy_per_child
            * children
            * p_school_meals.school_days
        )



from openfisca_us.model_api import *


class tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    definition_period = YEAR
    label = "TANF countable income"
    documentation = "Countable income for calculating Temporary Assistance for Needy Families benefit."
    unit = USD

    def formula(spm_unit, period, parameters):
        earned_income = spm_unit("tanf_gross_earned_income", period)
        state = spm_unit.household("state_code_str", period)
        earned_income_deduction = parameters(
            period
        ).hhs.tanf.cash.earned_income_deduction[state]
        countable_earned_income = earned_income * (1 - earned_income_deduction)
        # No deduction for unearned income.
        unearned_income = spm_unit("tanf_gross_unearned_income", period)
        return countable_earned_income + unearned_income



from openfisca_us.model_api import *


class spm_unit_benefits(Variable):
    value_type = float
    entity = SPMUnit
    label = "Benefits"
    definition_period = YEAR
    unit = USD

    def formula(spm_unit, period, parameters):
        PERSON_COMPONENTS = [
            "ssdi",
            "ca_cvrp",  # California Clean Vehicle Rebate Project.
            "gi_cash_assistance",
        ]
        SPMU_COMPONENTS = [
            "snap",
            "school_meal_subsidy",
            "ssi",
            "lifeline",
            # "tanf", # Exclude until defined for California.
        ]
        person_components = aggr(spm_unit, period, PERSON_COMPONENTS)
        spmu_components = add(spm_unit, period, SPMU_COMPONENTS)
        return person_components + spmu_components



from openfisca_us.model_api import *


class eitc_maximum(Variable):
    value_type = float
    entity = TaxUnit
    label = "Maximum EITC"
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/32#a"
    unit = USD

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        num_children = tax_unit.sum(person("is_child", period))
        eitc = parameters(period).irs.credits.eitc
        maximum = eitc.max.calc(num_children)
        phased_in_rate = eitc.phase_in_rate.calc(num_children)
        earned_income = tax_unit("filer_earned", period)
        phased_in_amount = earned_income * phased_in_rate
        return min_(maximum, phased_in_amount)



from openfisca_us.model_api import *
from openfisca_us.variables.demographic.tax_unit.MARSType import MARSType


class dsi(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    documentation = (
        "True if claimed as dependent on another return; otherwise false"
    )


class eic(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Number of EIC qualifying children (range: 0 to 3)"


class flpdyr(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Calendar year for which taxes are calculated"


class mars(Variable):
    value_type = Enum
    entity = TaxUnit
    possible_values = MARSType
    default_value = MARSType.SINGLE
    definition_period = YEAR
    label = "Marital status for the tax unit"

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        spouse_with_age = person("is_tax_unit_spouse", period)
        has_age = person("age", period) > 0
        has_spouse_with_age = tax_unit.any(spouse_with_age & has_age)
        return where(has_spouse_with_age, MARSType.JOINT, MARSType.SINGLE)


class midr(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    documentation = (
        "True if separately filing spouse itemizes; otherwise false"
    )


class recid(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Unique numeric identifier for filing unit; appears as RECID variable in tc CLI minimal output"


class xtot(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Total number of exemptions for filing unit"


class age_head(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Age in years of taxpayer (i.e. primary adult)"
    unit = "year"

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        age = person("age", period)
        head = person("is_tax_unit_head", period)
        return tax_unit.max(age * head)


class age_spouse(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Age in years of spouse (i.e. secondary adult if present)"
    unit = "year"

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        age = person("age", period)
        spouse = person("is_tax_unit_spouse", period)
        return tax_unit.max(age * spouse)


class agi_bin(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Historical AGI category used in data extrapolation"


class blind_head(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    documentation = "True if taxpayer is blind; otherwise False"


class blind_spouse(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    documentation = "1 if spouse is blind; otherwise 0"


class cmbtp(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Estimate of income on (AMT) Form 6251 but not in AGI"
    unit = USD


class filer_cmbtp(Variable):
    value_type = float
    entity = TaxUnit
    label = "Income on Form 6251 not in AGI for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("cmbtp", tax_unit, period)


class e00200(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Wages, salaries, and tips net of pension contributions"
    unit = USD

    def formula(person, period, parameters):
        return person("employment_income", period)


class filer_e00200(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Wages, salaries, and tips for filing unit (excluding dependents) net of pension contributions (pencon)"
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e00200", tax_unit, period)


class pencon(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Contributions to defined-contribution pension plans"
    unit = USD


class filer_pencon(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Contributions to defined-contribution pension plans for filing unit (excluding dependents)"
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("pencon", tax_unit, period)


class e00300(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Taxable interest income"
    unit = USD

    def formula(person, period, parameters):
        return person("interest_income", period)


class filer_e00300(Variable):
    value_type = float
    entity = TaxUnit
    label = "Taxable interest income for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e00300", tax_unit, period)


taxable_interest = variable_alias("taxable_interest", e00300)


class e00400(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Tax-exempt interest income"
    unit = USD


class filer_e00400(Variable):
    value_type = float
    entity = TaxUnit
    label = (
        "Tax-exempt interest income for the tax unit (excluding dependents)"
    )
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e00400", tax_unit, period)


tax_exempt_interest = variable_alias("tax_exempt_interest", e00400)


class e00600(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Ordinary dividends included in AGI"
    unit = USD

    def formula(person, period, parameters):
        return person("dividend_income", period)


class filer_e00600(Variable):
    value_type = float
    entity = TaxUnit
    label = "Ordinary dividends for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e00600", tax_unit, period)


class e00650(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Qualified dividends included in ordinary dividends"
    unit = USD


class filer_e00650(Variable):
    value_type = float
    entity = TaxUnit
    label = "Qualified dividends included in ordinary dividends for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e00650", tax_unit, period)


class e00700(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Taxable refunds of state and local income taxes"
    unit = USD


class filer_e00700(Variable):
    value_type = float
    entity = TaxUnit
    label = "Taxable refunds of state/local taxes for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e00700", tax_unit, period)


class e00800(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Alimony received"
    unit = USD


class filer_e00800(Variable):
    value_type = float
    entity = TaxUnit
    label = "Alimony for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e00800", tax_unit, period)


class e00900(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Sch C business net profit/loss"
    unit = USD

    def formula(person, period, parameters):
        return person("self_employment_income", period)


class filer_e00900(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = (
        "Sch C business net profit/loss for filing unit (excluding dependents)"
    )
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e00900", tax_unit, period)


class e01100(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Capital gain distributions not reported on Sch D"
    unit = USD


class filer_e01100(Variable):
    value_type = float
    entity = TaxUnit
    label = "Capital gains not reported on Sch D for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e01100", tax_unit, period)


class e01200(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Other net gain/loss from Form 4797"
    unit = USD


class filer_e01200(Variable):
    value_type = float
    entity = TaxUnit
    label = "Other net gain/loss for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e01200", tax_unit, period)


class e01400(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Taxable IRA distributions"
    unit = USD


class filer_e01400(Variable):
    value_type = float
    entity = TaxUnit
    label = "Taxable IRA distributions for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e01400", tax_unit, period)


class e01500(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Total pensions and annuities"
    unit = USD


class filer_e01500(Variable):
    value_type = float
    entity = TaxUnit
    label = "Pensions and annuities for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e01500", tax_unit, period)


class e01700(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Taxable pensions and annuities"
    unit = USD


class filer_e01700(Variable):
    value_type = float
    entity = TaxUnit
    label = "Taxable pensions and annuities for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e01700", tax_unit, period)


class e02000(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Sch E total rental, royalty, partnership, S-corporation, etc, income/loss (includes e26270 and e27200)"
    unit = USD


class filer_e02000(Variable):
    value_type = float
    entity = TaxUnit
    label = "Rentals, royalties etc. for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e02000", tax_unit, period)


filer_rental_income = variable_alias("filer_rental_income", filer_e02000)


class e02100(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Farm net income/loss from Sch F"
    unit = USD


class filer_e02100(Variable):
    value_type = float
    entity = TaxUnit
    label = "Farm net income/loss for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e02100", tax_unit, period)


class e02300(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Unemployment insurance benefits"
    unit = USD


class filer_e02300(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Unemployment insurance benefits for filing unit (excluding dependents)"
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e02300", tax_unit, period)


class e02400(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Total social security (OASDI) benefits"
    unit = USD


class filer_e02400(Variable):
    value_type = float
    entity = TaxUnit
    label = "Social security benefits for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e02400", tax_unit, period)


class e03150(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Total deductible IRA contributions"
    unit = USD


class filer_e03150(Variable):
    value_type = float
    entity = TaxUnit
    label = (
        "Deductible IRA contributions for the tax unit (excluding dependents)"
    )
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03150", tax_unit, period)


class e03210(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Student loan interest"
    unit = USD


class filer_e03210(Variable):
    value_type = float
    entity = TaxUnit
    label = "Student loan interest for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03210", tax_unit, period)


class e03220(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Educator expenses"
    unit = USD


class filer_e03220(Variable):
    value_type = float
    entity = TaxUnit
    label = "Educator expenses for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03220", tax_unit, period)


class e03230(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Tuition and fees from Form 8917"
    unit = USD


class filer_e03230(Variable):
    value_type = float
    entity = TaxUnit
    label = "Tuition for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03230", tax_unit, period)


class e03240(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Domestic production activities from Form 8903"
    unit = USD


class filer_e03240(Variable):
    value_type = float
    entity = TaxUnit
    label = "Domestic production activities for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03240", tax_unit, period)


class e03270(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Self-employed health insurance deduction"
    unit = USD


class filer_e03270(Variable):
    value_type = float
    entity = TaxUnit
    label = "Self-employed health insurance for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03270", tax_unit, period)


class e03290(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Health savings account deduction from Form 8889"
    unit = USD


class filer_e03290(Variable):
    value_type = float
    entity = TaxUnit
    label = "Health savings account deduction for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03290", tax_unit, period)


class e03300(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Contributions to SEP, SIMPLE and qualified plans"
    unit = USD


class filer_e03300(Variable):
    value_type = float
    entity = TaxUnit
    label = "SEP, SIMPLE, etc. contributions for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03300", tax_unit, period)


class e03400(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Penalty on early withdrawal of savings"
    unit = USD


class filer_e03400(Variable):
    value_type = float
    entity = TaxUnit
    label = "Early savings withdrawal penalty for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03400", tax_unit, period)


class e03500(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Alimony paid"
    unit = USD


class filer_e03500(Variable):
    value_type = float
    entity = TaxUnit
    label = "Alimony paid for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e03500", tax_unit, period)


class e07240(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Retirement savings contributions credit from Form 8880"
    unit = USD


retirement_savings_credit = variable_alias("retirement_savings_credit", e07240)


class e07260(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Residential energy credit from Form 5695"
    unit = USD


residential_energy_credit = variable_alias("residential_energy_credit", e07260)


class e07300(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Foreign tax credit from Form 1116"
    unit = USD


foreign_tax_credit = variable_alias("foreign_tax_credit", e07300)


class e07400(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "General business credit from Form 3800"
    unit = USD


class e07600(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Prior year minimum tax credit from Form 8801"
    unit = USD


class e09700(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Recapture of Investment Credit"
    unit = USD


class e09800(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Unreported payroll taxes from Form 4137 or 8919"
    unit = USD


class e09900(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Penalty tax on qualified retirement plans"
    unit = USD


class e11200(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Excess payroll (FICA/RRTA) tax withheld"
    unit = USD


class e17500(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Itemizable medical and dental expenses.  WARNING: this variable is zero below the floor in PUF data."
    unit = USD


class filer_e17500(Variable):
    value_type = float
    entity = TaxUnit
    label = "Itemized medical and dental expenses for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e17500", tax_unit, period)


class e18400(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Itemizable state and local income/sales taxes"
    unit = USD


class filer_e18400(Variable):
    value_type = float
    entity = TaxUnit
    label = "Itemized SALT for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e18400", tax_unit, period)


class e18500(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Itemizable real-estate taxes paid"
    unit = USD


class filer_e18500(Variable):
    value_type = float
    entity = TaxUnit
    label = "Itemized real estate for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e18500", tax_unit, period)


class e19200(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Itemizable interest paid"
    unit = USD


class filer_e19200(Variable):
    value_type = float
    entity = TaxUnit
    label = "Itemizable interest paid for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e19200", tax_unit, period)


class e19800(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Itemizable charitable giving: cash/check contributions.  WARNING: this variable is already capped in PUF data."
    unit = USD


class filer_e19800(Variable):
    value_type = float
    entity = TaxUnit
    label = "Itemized charity for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e19800", tax_unit, period)


class e20100(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Itemizable charitable giving: other than cash/check contributions.  WARNING: this variable is already capped in PUF data."
    unit = USD


class filer_e20100(Variable):
    value_type = float
    entity = TaxUnit
    label = "Itemized non-cash charity for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e20100", tax_unit, period)


class e20400(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Itemizable miscellaneous deductions.  WARNING: this variable is zero below the floor in PUF data."
    unit = USD


class filer_e20400(Variable):
    value_type = float
    entity = TaxUnit
    label = "Itemized misc. for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e20400", tax_unit, period)


class g20500(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = (
        "Itemizable gross (before 10% AGI disregard) casualty or theft loss"
    )
    unit = USD


class filer_g20500(Variable):
    value_type = float
    entity = TaxUnit
    label = "Itemized casualty loss for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("g20500", tax_unit, period)


class e24515(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Sch D: Un-Recaptured Section 1250 Gain"
    unit = USD


class filer_e24515(Variable):
    value_type = float
    entity = TaxUnit
    label = "Section 1250 Gain (un-recaptured) for the tax unit (excluding dependents)"
    unit = USD
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e24515", tax_unit, period)


class e24518(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Sch D: 28% Rate Gain or Loss"
    unit = USD


class filer_e24518(Variable):
    value_type = float
    entity = TaxUnit
    label = "Sch D Rate Gain or Loss for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e24518", tax_unit, period)


class e26270(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    unit = USD
    documentation = "Sch E: Combined partnership and S-corporation net income/loss (includes k1bx14p and k1bx14s amounts and is included in e02000)"


class filer_e26270(Variable):
    value_type = float
    entity = TaxUnit
    label = "Partnership/S-corp income for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e26270", tax_unit, period)


filer_partnership_s_corp_income = variable_alias(
    "filer_partnership_s_corp_income", filer_e26270
)


class e27200(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    unit = USD
    documentation = "Sch E: Farm rent net income or loss (included in e02000)"


class filer_e27200(Variable):
    value_type = float
    entity = TaxUnit
    label = "Label for the tax unit (excluding dependents)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e27200", tax_unit, period)


class e32800(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    unit = USD
    documentation = (
        "Child/dependent-care expenses for qualifying persons from Form 2441"
    )


class filer_e32800(Variable):
    value_type = float
    entity = TaxUnit
    label = (
        "Child/dependent-care expenses for the tax unit (excluding dependents)"
    )
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e32800", tax_unit, period)


class e58990(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    unit = USD
    documentation = "Investment income elected amount from Form 4952"


class filer_e58990(Variable):
    value_type = float
    entity = TaxUnit
    label = (
        "Investment income (Form 3952) for the tax unit (excluding dependents)"
    )
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e58990", tax_unit, period)


class e62900(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    unit = USD
    documentation = "Alternative Minimum Tax foreign tax credit from Form 6251"


class filer_e62900(Variable):
    value_type = float
    entity = TaxUnit
    label = "AMT foreign tax credit (Form 6251) for the tax unit (excluding dependents)"
    unit = USD
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("e62900", tax_unit, period)


class e87530(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    label = "Qualified tuition expenses"
    documentation = (
        "Adjusted qualified lifetime learning expenses for all students"
    )
    unit = USD


qualified_tuition_expenses = variable_alias(
    "qualified_tuition_expenses", e87530
)


class elderly_dependents(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Number of dependents age 65+ in filing unit excluding taxpayer and spouse"


class f2441(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Number of child/dependent-care qualifying persons"


class f6251(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    documentation = (
        "True if Form 6251 (AMT) attached to return; otherwise false"
    )


class a_lineno(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "CPS line number for the person record of the head of the tax filing unit (not used in tax-calculation logic)"


class ffpos(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "CPS family identifier within household (not used in tax-calculation logic)"


class fips(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "FIPS state code (not used in tax-calculation logic)"


class h_seq(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = (
        "CPS household sequence number (not used in tax-calculation logic)"
    )


class data_source(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    documentation = "True if unit is created primarily from IRS-SOI PUF data; false if created primarily from CPS data (not used in tax-calculation logic)"


class k1bx14(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = (
        "Partner self-employment earnings/loss (included in e26270 total)"
    )
    unit = USD


class filer_k1bx14(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Partner self-employment earnings/loss for tax unit (excluding dependents) (included in e26270 total)"
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("k1bx14", tax_unit, period)


class n24(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Number of children who are Child-Tax-Credit eligible, one condition for which is being under age 17"


class nu06(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Number of dependents under 6 years old"


class nu13(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Number of dependents under 13 years old"


class nu18(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Number of people under 18 years old in the filing unit"


class n1820(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Number of people age 18-20 years old in the filing unit"


class n21(Variable):
    value_type = int
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Number of people 21 years old or older in the filing unit"


class p08000(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Other tax credits (but not including Sch R credit)"
    unit = USD


class p22250(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Sch D: Net short-term capital gains/losses"
    unit = USD


class filer_p22250(Variable):
    value_type = float
    entity = TaxUnit
    label = (
        "Net short-term capital gains for the tax unit (excluding dependents)"
    )
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("p22250", tax_unit, period)


class p23250(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Sch D: Net long-term capital gains/losses"
    unit = USD


class filer_p23250(Variable):
    value_type = float
    entity = TaxUnit
    label = (
        "Net long-term capital gains for the tax unit (excluding dependents)"
    )
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        return tax_unit_non_dep_sum("p23250", tax_unit, period)


class e87521(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Total tentative AmOppCredit amount for all students"
    unit = USD


class s006(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Filing unit sampling weight; appears as WEIGHT variable in tc CLI minimal output"


class pt_sstb_income(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Whether business income is from a specified service trade or business (SSTB), rather than from a qualified trade or business"


class pt_binc_w2_wages(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Filing unit's share of total W-2 wages paid by the pass-through business"
    unit = "/1"


class pt_ubia_property(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Filing unit's share of total business property owned by the pass-through business"
    unit = "/1"



from openfisca_us.model_api import *


class ssdi(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Social Security Disability Insurance amount"
    label = "Social Security Disability Insurance"
    unit = USD



from openfisca_us.model_api import *


class MedicaidPersonType(Enum):
    ADULT_WITHOUT_DEPENDENT = "Adult without dependent"
    ADULT_WITH_DEPENDENT = "Adult with dependent"
    CHILD_AGE_0 = "Child age 0"
    CHILD_AGE_1_5 = "Child age 1 to 5"
    CHILD_AGE_6_18 = "Child age 6 to 18"


class medicaid_person_type(Variable):
    value_type = Enum
    possible_values = MedicaidPersonType
    default_value = MedicaidPersonType.ADULT_WITHOUT_DEPENDENT
    entity = Person
    definition_period = YEAR
    label = "Medicaid person type"
    documentation = "Person type for Medicaid"

    def formula(person, period, parameters):
        # Get the person's age.
        age = person("age", period)
        # Get the existence of dependents, as defined by people 18 or younger.
        has_dependents = person.spm_unit.any(
            person.spm_unit.members("age", period) < 19
        )
        return select(
            [age == 0, age < 6, age < 19, has_dependents, True],
            [
                MedicaidPersonType.CHILD_AGE_0,
                MedicaidPersonType.CHILD_AGE_1_5,
                MedicaidPersonType.CHILD_AGE_6_18,
                MedicaidPersonType.ADULT_WITH_DEPENDENT,
                MedicaidPersonType.ADULT_WITHOUT_DEPENDENT,
            ],
        )



from openfisca_us.model_api import *


class eitc_reduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "EITC reduction"
    unit = USD
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/32#a_2"

    def formula(tax_unit, period, parameters):
        eitc = parameters(period).irs.credits.eitc
        earnings = tax_unit("filer_earned", period)
        highest_income_variable = max_(
            earnings, tax_unit("adjusted_gross_income", period)
        )
        is_joint = tax_unit("tax_unit_is_joint", period)
        num_children = tax_unit.sum(tax_unit.members("is_child", period))
        phaseout_start = (
            eitc.phaseout.start.calc(num_children)
            + is_joint * eitc.phaseout.joint_bonus
        )
        uncapped_reduction = max_(0, highest_income_variable - phaseout_start)
        return min_(uncapped_reduction, tax_unit("eitc_maximum", period))



from openfisca_us.model_api import *


class ssi_amount_if_eligible(Variable):
    value_type = float
    entity = SPMUnit
    definition_period = YEAR
    documentation = (
        "Supplemental Security Income amount if someone is eligible"
    )
    label = "SSI amount if eligible"
    unit = USD



from openfisca_us.model_api import *


class broadband_cost(Variable):
    value_type = float
    entity = SPMUnit
    label = "Broadband cost"
    documentation = "Broadband cost for this SPM unit"
    unit = USD
    definition_period = YEAR



from openfisca_us.model_api import *


class tanf_initial_employment_deduction(Variable):
    value_type = float
    entity = SPMUnit
    definition_period = YEAR
    label = "TANF IED (Initial Employment Deduction)"
    documentation = "The amount deducted from the countable earnings of a TANF application when calculating initial eligibility."
    unit = USD

    def formula(spm_unit, period, parameters):
        family_size = spm_unit("spm_unit_size", period).astype(str)
        state = spm_unit.household("state_code_str", period)
        ied = parameters(period).hhs.tanf.cash.initial_employment_deduction
        return ied[state][family_size] * 12



from openfisca_us.model_api import *


class is_tax_unit_spouse(Variable):
    value_type = bool
    entity = Person
    label = "Spouse of tax unit"
    definition_period = YEAR

    def formula(person, period, parameters):
        # Establish basic conditions
        is_adult = person("age", period) >= 18
        is_not_head = ~person("is_tax_unit_head", period)
        eligible = is_adult * is_not_head
        # Of those who meet the criteria, select the first person defined
        rank = person.tax_unit.members_position * eligible
        highest_rank = person.tax_unit.sum(rank)
        return eligible & (rank == highest_rank)
